@@ -95,7 +95,7 @@ char *tracker_service_types[] = {
 "Appointments",
 "Tasks",
 "Bookmarks",
-"History",
+"WebHistory",
 "Projects",
 NULL
 };
@@ -392,6 +392,34 @@ tracker_get_stats (TrackerClient *client,  GError **error)
 }
 
 
+void
+tracker_set_bool_option (TrackerClient *client, const char *option, gboolean value, GError **error)
+{
+	org_freedesktop_Tracker_set_bool_option (client->proxy, option, value,  &*error);
+}
+
+void
+tracker_set_int_option (TrackerClient *client, const char *option, int value, GError **error)
+{
+	org_freedesktop_Tracker_set_int_option (client->proxy, option, value,  &*error);
+}
+
+
+void
+tracker_shutdown (TrackerClient *client, gboolean reindex, GError **error)
+{
+	org_freedesktop_Tracker_shutdown (client->proxy, reindex,  &*error);
+}
+
+
+void
+tracker_prompt_index_signals (TrackerClient *client, GError **error)
+{
+	org_freedesktop_Tracker_prompt_index_signals (client->proxy, &*error);
+}
+
+
+
 char **		
 tracker_metadata_get (TrackerClient *client, ServiceType service, const char *id, char **keys, GError **error) 
 {
@@ -560,6 +588,22 @@ tracker_keywords_search	(TrackerClient *client, int live_query_id, ServiceType s
 }
 
 
+int
+tracker_search_get_hit_count (TrackerClient *client, ServiceType service, const char *search_text, GError **error)
+{
+
+	int count = 0;
+	char *service_str = tracker_service_types[service];
+
+	if (!org_freedesktop_Tracker_Search_get_hit_count (client->proxy_search, service_str, search_text, &count, &*error)) {
+		return 0;
+	}
+
+	return count;
+
+}
+
+
 GPtrArray *
 tracker_search_get_hit_count_all (TrackerClient *client, const char *search_text, GError **error)
 {
@@ -897,6 +941,62 @@ tracker_get_stats_async	(TrackerClient *client,  TrackerGPtrArrayReply callback,
 }
 
 
+
+void
+tracker_set_bool_option_async (TrackerClient *client, const char *option, gboolean value, TrackerVoidReply callback, gpointer user_data)
+{
+	VoidCallBackStruct *callback_struct;
+
+	callback_struct = g_new (VoidCallBackStruct, 1);
+	callback_struct->callback = callback;
+	callback_struct->data = user_data;
+
+	client->last_pending_call = org_freedesktop_Tracker_set_bool_option_async  (client->proxy, option, value, tracker_void_reply, callback_struct);
+}
+
+void
+tracker_set_int_option_async (TrackerClient *client, const char *option, int value, TrackerVoidReply callback, gpointer user_data)
+{
+	VoidCallBackStruct *callback_struct;
+
+	callback_struct = g_new (VoidCallBackStruct, 1);
+	callback_struct->callback = callback;
+	callback_struct->data = user_data;
+
+	client->last_pending_call = org_freedesktop_Tracker_set_int_option_async  (client->proxy, option, value, tracker_void_reply, callback_struct);
+}
+
+
+void
+tracker_shutdown_async (TrackerClient *client, gboolean reindex, TrackerVoidReply callback, gpointer user_data)
+{
+	VoidCallBackStruct *callback_struct;
+
+	callback_struct = g_new (VoidCallBackStruct, 1);
+	callback_struct->callback = callback;
+	callback_struct->data = user_data;
+
+	client->last_pending_call = org_freedesktop_Tracker_shutdown_async  (client->proxy, reindex, tracker_void_reply, callback_struct);
+}
+
+
+void
+tracker_prompt_index_signals_async (TrackerClient *client, TrackerVoidReply callback, gpointer user_data)
+{
+	VoidCallBackStruct *callback_struct;
+
+	callback_struct = g_new (VoidCallBackStruct, 1);
+	callback_struct->callback = callback;
+	callback_struct->data = user_data;
+
+	client->last_pending_call = org_freedesktop_Tracker_prompt_index_signals_async  (client->proxy, tracker_void_reply, callback_struct);
+}
+
+
+
+
+
+
 void	
 tracker_metadata_get_async (TrackerClient *client, ServiceType service, const char *id, char **keys, TrackerArrayReply callback, gpointer user_data)
 {
@@ -1105,6 +1205,21 @@ tracker_keywords_search_async	(TrackerClient *client, int live_query_id, Service
 }
 
 
+void		
+tracker_search_text_get_hit_count_async (TrackerClient *client, ServiceType service, const char *search_text, TrackerIntReply callback, gpointer user_data)
+{
+	char *service_str = tracker_service_types[service];
+
+	IntCallBackStruct *callback_struct;
+
+	callback_struct = g_new (IntCallBackStruct, 1);
+	callback_struct->callback = callback;
+	callback_struct->data = user_data;
+
+	client->last_pending_call =  org_freedesktop_Tracker_Search_get_hit_count_async (client->proxy_search, service_str, search_text, tracker_int_reply, callback_struct);
+
+
+}
 
 
 void
@@ -1423,8 +1538,3 @@ tracker_search_metadata_by_text_and_location_async (TrackerClient *client, const
 
 
 
-
-
-
-
-
