@@ -19,22 +19,32 @@
 
 #include "config.h"
 
-#ifdef HAVE_EXEMPI
-
 #include <glib.h>
 
 #include "tracker-extract.h"
+#include "tracker-xmp.h"
 
-void tracker_extract_xmp (gchar* filename, GHashTable *metadata)
+
+static void
+tracker_extract_xmp (const gchar* filename, GHashTable *metadata)
 {
 	gchar *contents;
 	gsize length;
 	GError *error;
 
-	if ( g_file_get_contents( filename, &contents, &length, &error ) )
-		tracker_read_xmp(contents, length, metadata);
+	if ( g_file_get_contents ( filename, &contents, &length, &error ) )
+		tracker_read_xmp (contents, length, metadata);
 }
 
-#else
-#warning "Not building XMP metadata extractor."
-#endif  /* HAVE_EXEMPI */
+
+TrackerExtractorData data[] = {
+	{ "application/rdf+xml", tracker_extract_xmp },
+	{ NULL, NULL }
+};
+
+
+TrackerExtractorData *
+tracker_get_extractor_data (void)
+{
+	return data;
+}
