@@ -32,17 +32,13 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <glib.h>
+#include <gmodule.h>
 
 #include "tracker-extract.h"
 
 #define MAX_MEM 128
 #define MAX_MEM_AMD64 512
 
-#ifdef HAVE_EXEMPI
-#include <exempi/xmp.h>
-#include <exempi/xmpconsts.h>
-#endif
-
 typedef enum {
 	IGNORE_METADATA,
 	NO_METADATA,
@@ -53,118 +49,7 @@ typedef enum {
 } MetadataFileType;
 
 
-
-typedef void (*MetadataExtractFunc)(gchar *, GHashTable *);
- 
-typedef struct {
- 	char			*mime;
- 	MetadataExtractFunc	extractor;
-} MimeToExtractor;
-
-void tracker_extract_mplayer	(gchar *, GHashTable *);
-void tracker_extract_totem	(gchar *, GHashTable *);
-void tracker_extract_oasis	(gchar *, GHashTable *);
-void tracker_extract_ps		(gchar *, GHashTable *);
-#ifdef HAVE_LIBXML2
-void tracker_extract_html       (gchar *, GHashTable *);
-#endif
-#ifdef HAVE_POPPLER
-void tracker_extract_pdf	(gchar *, GHashTable *);
-#endif
-void tracker_extract_abw	(gchar *, GHashTable *);
-#ifdef HAVE_LIBGSF
-void tracker_extract_msoffice	(gchar *, GHashTable *);
-#endif
-#ifdef HAVE_EXEMPI
-void tracker_extract_xmp	(gchar *, GHashTable *);
-#endif
-void tracker_extract_mp3	(gchar *, GHashTable *);
-#ifdef HAVE_VORBIS
-void tracker_extract_vorbis	(gchar *, GHashTable *);
-#endif
-void tracker_extract_png	(gchar *, GHashTable *);
-#ifdef HAVE_LIBEXIF
-void tracker_extract_exif	(gchar *, GHashTable *);
-#endif
-void tracker_extract_imagemagick (gchar *, GHashTable *);
-#ifdef HAVE_GSTREAMER
-void tracker_extract_gstreamer	(gchar *, GHashTable *);
-#else
-#ifdef HAVE_LIBXINE
-void tracker_extract_xine	(gchar *, GHashTable *);
-#else
-#ifdef USING_EXTERNAL_VIDEO_PLAYER
-
-#endif
-#endif
-#endif
-  
-MimeToExtractor extractors[] = {
-	/* Document extractors */
- 	{ "application/vnd.oasis.opendocument.*",	tracker_extract_oasis		},
- 	{ "application/postscript",			tracker_extract_ps		},
-#ifdef HAVE_LIBXML2
- 	{ "text/html",                                  tracker_extract_html            },
- 	{ "application/xhtml+xml",                      tracker_extract_html            },
-#endif
-#ifdef HAVE_POPPLER
- 	{ "application/pdf",				tracker_extract_pdf		},
-#endif
-	{ "application/x-abiword",			tracker_extract_abw		},
-#ifdef HAVE_LIBGSF
- 	{ "application/msword",				tracker_extract_msoffice	},
- 	{ "application/vnd.ms-*",			tracker_extract_msoffice	},
-#endif
-#ifdef HAVE_EXEMPI
- 	{ "application/rdf+xml",			tracker_extract_xmp		},
-#endif
-
-  	/* Video extractors */
-#ifdef HAVE_GSTREAMER
- 	{ "video/*",					tracker_extract_gstreamer	},
- 	{ "video/*",					tracker_extract_mplayer		},
- 	{ "video/*",					tracker_extract_totem		},
-#else
-#ifdef HAVE_LIBXINE
- 	{ "video/*",					tracker_extract_xine		},
- 	{ "video/*",					tracker_extract_mplayer		},
- 	{ "video/*",					tracker_extract_totem		},
-#else
-#ifdef USING_EXTERNAL_VIDEO_PLAYER
- 	{ "video/*",					tracker_extract_mplayer		},
- 	{ "video/*",					tracker_extract_totem		},
-#endif
-#endif
-#endif
-  
-  	/* Audio extractors */
-	
-#ifdef HAVE_GSTREAMER
-	{ "audio/*",					tracker_extract_gstreamer	},
- 	{ "audio/*",					tracker_extract_mplayer		},
-
-#else
-#ifdef HAVE_LIBXINE
-	{ "audio/*",					tracker_extract_xine		},
- 	{ "audio/*",					tracker_extract_mplayer		},
-#else
-#ifdef USING_EXTERNAL_VIDEO_PLAYER
- 	{ "audio/*",					tracker_extract_mplayer		},
- 	{ "audio/*",					tracker_extract_totem		},
-#endif
-#endif
-#endif
-  
-	{ "audio/mp3",					tracker_extract_mp3		},
-
-     /* Image extractors */
-	{ "image/png",					tracker_extract_png		},
-#ifdef HAVE_LIBEXIF
-	{ "image/jpeg",					tracker_extract_exif		},
-#endif
-	{ "image/*",					tracker_extract_imagemagick	},
-	{ "",						NULL				}
-};
+GArray *extractors = NULL;
 
 
 static gboolean
@@ -466,7 +351,7 @@ tracker_child_cb (gpointer user_data)
 		g_printerr ("Error trying to set resource limit for cpu\n");
 	}
 
-	set_memory_rlimits();
+	set_memory_rlimits ();
 
 	/* Set child's niceness to 19 */
         errno = 0;
@@ -499,205 +384,94 @@ tracker_spawn (gchar **argv, gint timeout, gchar **tmp_stdout, gint *exit_status
 }
 
 
-#ifdef HAVE_EXEMPI
-void
-tracker_append_string_to_hash_table (GHashTable *metadata, const gchar *key, const gchar *value, gboolean append)
+static void
+initialize_extractors (void)
 {
-	gchar *new_value;
+	GDir        *dir;
+	GError      *error = NULL;
+	const gchar *name;
+	GArray      *generic_extractors = NULL;
 
-	if (append) {
-		gchar *orig;
-		if (g_hash_table_lookup_extended (metadata, key, NULL, (gpointer)&orig )) {
-			new_value = g_strconcat (orig, " ", value, NULL);
-		} else {
-			new_value = g_strdup (value);
-		}
-	} else {
-		new_value = g_strdup (value);
+	if (extractors != NULL)
+		return;
+
+	if (!g_module_supported ()) {
+		g_error ("Modules are not supported for this platform");
+		return;
 	}
 
-	g_hash_table_insert (metadata, g_strdup(key), new_value);
-}
+	extractors = g_array_sized_new (FALSE, TRUE,
+					sizeof (TrackerExtractorData),
+					10);
 
+	/* This array is going to be used to store
+	 * temporarily extractors with mimetypes such as "audio / *"
+	 */
+	generic_extractors = g_array_sized_new (FALSE, TRUE,
+						sizeof (TrackerExtractorData),
+						10);
 
-void tracker_xmp_iter (XmpPtr xmp, XmpIteratorPtr iter, GHashTable *metadata, gboolean append);
-void tracker_xmp_iter_simple (GHashTable *metadata, const gchar *schema, const gchar *path, const gchar *value, gboolean append);
+	dir = g_dir_open (MODULES_DIR, 0, &error);
 
-
-/* We have an array, now recursively iterate over it's children.  Set 'append' to true so that all values of the array are added
-   under one entry. */
-void
-tracker_xmp_iter_array (XmpPtr xmp, GHashTable *metadata, const gchar *schema, const gchar *path)
-{
-		XmpIteratorPtr iter = xmp_iterator_new (xmp, schema, path, XMP_ITER_JUSTCHILDREN);
-		tracker_xmp_iter (xmp, iter, metadata, TRUE);
-		xmp_iterator_free (iter);
-}
-
-
-/* We have an array, now recursively iterate over it's children.  Set 'append' to false so that only one item is used. */
-void
-tracker_xmp_iter_alt_text (XmpPtr xmp, GHashTable *metadata, const gchar *schema, const gchar *path)
-{
-		XmpIteratorPtr iter = xmp_iterator_new (xmp, schema, path, XMP_ITER_JUSTCHILDREN);
-		tracker_xmp_iter (xmp, iter, metadata, FALSE);
-		xmp_iterator_free (iter);
-}
-
-
-/* We have a simple element, but need to iterate over the qualifiers */
-void
-tracker_xmp_iter_simple_qual (XmpPtr xmp, GHashTable *metadata,
-                              const gchar *schema, const gchar *path, const gchar *value, gboolean append)
-{
-	XmpIteratorPtr iter = xmp_iterator_new(xmp, schema, path, XMP_ITER_JUSTCHILDREN | XMP_ITER_JUSTLEAFNAME);
-
-	XmpStringPtr the_path = xmp_string_new ();
-	XmpStringPtr the_prop = xmp_string_new ();
-
-	gchar *locale = setlocale (LC_ALL, NULL);
-	gchar *sep = strchr (locale,'.');
-	if (sep) {
-		locale[sep - locale] = '\0';
-	}
-	sep = strchr (locale, '_');
-	if (sep) {
-		locale[sep - locale] = '-';
+	if (!dir) {
+		g_error ("Error opening modules directory: %s\n", error->message);
+		g_error_free (error);
+                g_array_free (extractors, TRUE);
+                extractors = NULL;
+                g_array_free (generic_extractors, TRUE);
+		return;
 	}
 
-	gboolean ignore_element = FALSE;
+	while ((name = g_dir_read_name (dir)) != NULL) {
+		GModule                  *module;
+		gchar                    *module_path;
+		TrackerExtractorDataFunc func;
+		TrackerExtractorData     *data;
 
-	while (xmp_iterator_next (iter, NULL, the_path, the_prop, NULL)) {
-		const gchar *qual_path = xmp_string_cstr (the_path);
-		const gchar *qual_value = xmp_string_cstr (the_prop);
-
-		if (strcmp (qual_path, "xml:lang") == 0) {
-			/* is this a language we should ignore? */
-			if (strcmp (qual_value, "x-default") != 0 && strcmp (qual_value, "x-repair") != 0 && strcmp (qual_value, locale) != 0) {
-				ignore_element = TRUE;
-				break;
-			}
+		if (!g_str_has_suffix (name, "." G_MODULE_SUFFIX)) {
+			continue;
 		}
-	}
 
-	if (!ignore_element) {
-		tracker_xmp_iter_simple (metadata, schema, path, value, append);
-	}
+		module_path = g_build_filename (MODULES_DIR, name, NULL);
 
-	xmp_string_free (the_prop);
-	xmp_string_free (the_path);
+		module = g_module_open (module_path, G_MODULE_BIND_LOCAL);
 
-	xmp_iterator_free (iter);
-}
-
-
-/* We have a simple element. Add any metadata we know about to the hash table  */
-void
-tracker_xmp_iter_simple (GHashTable *metadata,
-                         const gchar *schema, const gchar *path, const gchar *value, gboolean append)
-{
-	gchar *name = g_strdup (strchr (path, ':')+1);
-	const gchar *index = strrchr (name, '[');
-	if (index) {
-		name[index-name] = '\0';
-	}
-
-	/* Dublin Core */
-	if (strcmp(schema, NS_DC) == 0) {
-		if (strcmp (name, "title") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Title", value, append);
+		if (!module) {
+			g_warning ("Could not load module: %s", name);
+			g_free (module_path);
+			continue;
 		}
-		else if (strcmp (name, "rights") == 0) {
-			tracker_append_string_to_hash_table (metadata, "File:Copyright", value, append);
-		}
-		else if (strcmp (name, "creator") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Creator", value, append);
-		}
-		else if (strcmp (name, "description") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Description", value, append);
-		}
-		else if (strcmp (name, "date") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Date", value, append);
-		}
-		else if (strcmp (name, "keywords") == 0) {
-			tracker_append_string_to_hash_table (metadata, "Image:Keywords", value, append);
-		}
-	}
-	/* Creative Commons */
-	else if (strcmp (schema, NS_CC) == 0) {
-		if (strcmp (name, "license") == 0) {
-			tracker_append_string_to_hash_table (metadata, "File:License", value, append);
-		}
-	}
 
-	free(name);
-}
+		g_module_make_resident (module);
 
+		if (g_module_symbol (module, "tracker_get_extractor_data", (gpointer *) &func)) {
+			data = (func) ();
 
-/* Iterate over the XMP, dispatching to the appropriate element type (simple, simple w/qualifiers, or an array) handler */
-void
-tracker_xmp_iter (XmpPtr xmp, XmpIteratorPtr iter, GHashTable *metadata, gboolean append)
-{
-	XmpStringPtr the_schema = xmp_string_new ();
-	XmpStringPtr the_path = xmp_string_new ();
-	XmpStringPtr the_prop = xmp_string_new ();
-
-	uint32_t opt;
-	while (xmp_iterator_next (iter, the_schema, the_path, the_prop, &opt)) {
-		const gchar *schema = xmp_string_cstr (the_schema);
-		const gchar *path = xmp_string_cstr (the_path);
-		const gchar *value = xmp_string_cstr (the_prop);
-
-		if (XMP_IS_PROP_SIMPLE (opt)) {
-			if (strcmp (path,"") != 0) {
-				if (XMP_HAS_PROP_QUALIFIERS (opt)) {
-					tracker_xmp_iter_simple_qual (xmp, metadata, schema, path, value, append);
+			while (data->mime) {
+				if (strchr (data->mime, '*') != NULL) {
+					g_array_append_val (generic_extractors, *data);
 				} else {
-					tracker_xmp_iter_simple (metadata, schema, path, value, append);
+					g_array_append_val (extractors, *data);
 				}
-			}	
-		}
-		else if (XMP_IS_PROP_ARRAY (opt)) {
-			if (XMP_IS_ARRAY_ALTTEXT (opt)) {
-				tracker_xmp_iter_alt_text (xmp, metadata, schema, path);
-				xmp_iterator_skip (iter, XMP_ITER_SKIPSUBTREE);
-			} else {
-				tracker_xmp_iter_array (xmp, metadata, schema, path);
-				xmp_iterator_skip (iter, XMP_ITER_SKIPSUBTREE);
+
+				data++;
 			}
 		}
+
+		g_free (module_path);
 	}
 
-	xmp_string_free (the_prop);
-	xmp_string_free (the_path);
-	xmp_string_free (the_schema);
-}
-#endif
+	/* append the generic extractors at the end of
+	 * the list, so the specific ones are used first
+	 */
+	g_array_append_vals (extractors, generic_extractors->data, generic_extractors->len);
 
-
-void
-tracker_read_xmp (const gchar *buffer, size_t len, GHashTable *metadata)
-{
-	#ifdef HAVE_EXEMPI
-	xmp_init ();
-
-	XmpPtr xmp = xmp_new_empty ();
-	xmp_parse (xmp, buffer, len);
-	if (xmp != NULL) {
-		XmpIteratorPtr iter = xmp_iterator_new (xmp, NULL, NULL, XMP_ITER_PROPERTIES);
-		tracker_xmp_iter (xmp, iter, metadata, FALSE);
-		xmp_iterator_free (iter);
-	
-		xmp_free (xmp);
-	}
-
-	xmp_terminate ();
-	#endif
+	g_array_free (generic_extractors, TRUE);
 }
 
 
 static GHashTable *
-tracker_get_file_metadata (const gchar *uri, gchar *mime)
+tracker_get_file_metadata (const gchar *uri, const gchar *mime)
 {
 	GHashTable      *meta_table;
 	gchar		*uri_in_locale;
@@ -720,24 +494,24 @@ tracker_get_file_metadata (const gchar *uri, gchar *mime)
 	meta_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	if (mime) {
-		MimeToExtractor *p;
+		guint i;
+		TrackerExtractorData *data;
 
-		for (p = extractors; p->extractor; ++p) {
-			if (g_pattern_match_simple (p->mime, mime)) {
-				(*p->extractor)(uri_in_locale, meta_table);
+		for (i = 0; i < extractors->len; i++) {
+			data = &g_array_index (extractors, TrackerExtractorData, i);
+
+			if (g_pattern_match_simple (data->mime, mime)) {
+				(*data->extractor) (uri_in_locale, meta_table);
 
 				if (g_hash_table_size (meta_table) == 0) {
 					continue;
 				}
 
 				g_free (uri_in_locale);
-				g_free (mime);
 
 				return meta_table;
 			}
 		}
-
-		g_free (mime);
 	}
 
 	g_free (uri_in_locale);
@@ -777,11 +551,12 @@ kill_app_timeout (void)
 	exit (EXIT_FAILURE);
 }
 
+
 gint
 main (gint argc, gchar *argv[])
 {
-	GHashTable	*meta;
-	gchar		*filename;
+	GHashTable *meta;
+	gchar	   *filename;
 
 	set_memory_rlimits ();
 
@@ -794,9 +569,11 @@ main (gint argc, gchar *argv[])
 
 	setlocale (LC_ALL, "");
 
-	if ((argc == 1) || (argc > 3)) {
+	initialize_extractors ();
+
+	if (argc == 1 || argc > 3) {
 		g_print ("usage: tracker-extract file [mimetype]\n");
-		return 0;
+		return EXIT_FAILURE;
 	}
 
 	filename = g_filename_to_utf8 (argv[1], -1, NULL, NULL, NULL);
@@ -811,11 +588,11 @@ main (gint argc, gchar *argv[])
 
 		if (!mime) {
 			g_warning ("locale to UTF8 failed for mime!");
-			return 1;
+			return EXIT_FAILURE;
 		}
 
-		/* mime will be free in tracker_get_file_metadata() */
 		meta = tracker_get_file_metadata (filename, mime);
+		g_free (mime);
 	} else {
 		meta = tracker_get_file_metadata (filename, NULL);
 	}
@@ -827,5 +604,5 @@ main (gint argc, gchar *argv[])
 		g_hash_table_destroy (meta);
 	}
 
-	return 0;
+	return EXIT_SUCCESS;
 }
