@@ -126,12 +126,12 @@ static char *search_service_types[] = {
 "Appointments",
 "Tasks",
 "Bookmarks",
-"History",
+"WebHistory",
 "Projects",
 NULL
 };
 
-static service_info_t services[12] = {
+static service_info_t services[13] = {
 
         { "Emails",        N_("Emails"),       "stock_mail",               NULL, SERVICE_EMAILS,            NULL, FALSE, 0, 0},
         { "Files",         N_("All Files"),    "system-file-manager",      NULL, SERVICE_FILES,             NULL, FALSE, 0, 0},
@@ -144,6 +144,7 @@ static service_info_t services[12] = {
         { "Development",   N_("Development"),  "applications-development", NULL, SERVICE_DEVELOPMENT_FILES, NULL, FALSE, 0, 0},
         { "Conversations", N_("Chat Logs"),    "stock_help-chat",          NULL, SERVICE_CONVERSATIONS,     NULL, FALSE, 0, 0},
         { "Applications",  N_("Applications"), "system-run",               NULL, SERVICE_APPLICATIONS,      NULL, FALSE, 0, 0},
+        { "WebHistory",    N_("WebHistory"),    "text-html",               NULL, SERVICE_WEBHISTORY,        NULL, FALSE, 0, 0},
         { NULL,            NULL,               NULL,                       NULL, -1,                        NULL, FALSE, 0, 0},
 };
 
@@ -269,7 +270,7 @@ display_dialog_character_set_conversion_error (GtkWidget * window,
 
 static void
 display_error_dialog (GtkWidget * window,
-		      GError * error)
+		      const char *error)
 {
 	GtkWidget * dialog;
 
@@ -277,12 +278,11 @@ display_error_dialog (GtkWidget * window,
 				 GTK_DIALOG_DESTROY_WITH_PARENT,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 _("The following error has occured :"));
+				 _("The following error has occurred :"));
 
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-					  (error == NULL) ? " " : error->message);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), error);
 
-	gtk_window_set_title (GTK_WINDOW (dialog), "Error");
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
 
@@ -302,7 +302,9 @@ start_animation (GSearchWindow * gsearch,
 		gsearch->focus = gtk_window_get_focus (GTK_WINDOW (gsearch->window));
 
 		gtk_widget_set_sensitive (gsearch->find_button, FALSE);
-		gtk_widget_set_sensitive (gsearch->search_results_save_results_as_item, FALSE);
+                if (gsearch->type < 10) {
+			gtk_widget_set_sensitive (gsearch->search_results_save_results_as_item, FALSE);
+		}
 		gtk_widget_set_sensitive (gsearch->search_results_vbox, TRUE);
 		gtk_widget_set_sensitive (GTK_WIDGET (gsearch->search_results_tree_view), TRUE);
 
@@ -316,7 +318,9 @@ stop_animation (GSearchWindow * gsearch)
 	gtk_window_set_default (GTK_WINDOW (gsearch->window), gsearch->find_button);
 	gtk_widget_set_sensitive (gsearch->name_and_folder_table, TRUE);
 	gtk_widget_set_sensitive (gsearch->find_button, TRUE);
-	gtk_widget_set_sensitive (gsearch->search_results_save_results_as_item, TRUE);
+        if (gsearch->type < 10) {
+		gtk_widget_set_sensitive (gsearch->search_results_save_results_as_item, TRUE);
+	}
 	gtk_widget_show (gsearch->find_button);
 	
 
@@ -517,6 +521,7 @@ add_file_to_search_results (const gchar * file,
 
 	if (!g_file_test (uri, G_FILE_TEST_EXISTS)) {
 		g_warning ("file %s does not exist", file);
+		g_free (uri);
 		return;
 	}
 
@@ -913,8 +918,8 @@ filename_cell_data_func (GtkTreeViewColumn * column,
 	gtk_tree_model_get (model, iter, COLUMN_PATH, &fpath, -1);
 	gtk_tree_model_get (model, iter, COLUMN_MIME, &type, -1);
 
-	gchar * display_name = crop_string (name, 40);
-	gchar * display_path = crop_string (fpath, 55);
+	gchar * display_name = crop_string (name, 65);
+	gchar * display_path = crop_string (fpath, 120);
 
 	gchar * mark_name = g_markup_escape_text (display_name, -1);
 	gchar * mark_dir =  g_markup_escape_text (display_path, -1);
@@ -1056,6 +1061,13 @@ create_search_results_section (GSearchWindow * gsearch)
 
 	/* Translators: this will appears as "Search results: no search performed" */
 	gsearch->count_label = gtk_label_new (_("no search performed"));
+	gtk_label_set_selectable (GTK_LABEL (gsearch->count_label), TRUE);
+	tracker_set_atk_relationship(gsearch->count_label,
+                                     ATK_RELATION_LABELLED_BY,
+                                     label);
+	tracker_set_atk_relationship(label, ATK_RELATION_LABEL_FOR,
+                                     gsearch->count_label);
+
 	gtk_box_pack_start (GTK_BOX (label_box), gsearch->count_label, FALSE, TRUE, 0);
 
 	button_next = gtk_button_new();
@@ -1090,7 +1102,6 @@ create_search_results_section (GSearchWindow * gsearch)
 	gtk_widget_set_sensitive (gsearch->back_button, FALSE);
 
 	gsearch->files_found_label = gtk_label_new (NULL);
-	gtk_label_set_selectable (GTK_LABEL (gsearch->files_found_label), TRUE);
 	gtk_box_pack_start (GTK_BOX (label_box), gsearch->files_found_label, FALSE, FALSE, 0);
 
 	window = gtk_scrolled_window_new (NULL, NULL);
@@ -1203,8 +1214,6 @@ create_search_results_section (GSearchWindow * gsearch)
 
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
-
-
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 
@@ -1214,9 +1223,6 @@ create_search_results_section (GSearchWindow * gsearch)
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (gsearch->search_results_tree_view), column);
 
-	gtk_tree_view_column_set_min_width (column, 200);
-	gtk_tree_view_column_set_max_width (column, 300);
-
 	/* create the snippet column */
 	renderer = gtk_cell_renderer_text_new ();
 
@@ -1231,7 +1237,11 @@ create_search_results_section (GSearchWindow * gsearch)
 						 gsearch, NULL);	
 
 	gtk_tree_view_column_set_reorderable (column, TRUE);
+
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_min_width (column, 0);
+	gtk_tree_view_column_set_max_width (column, 10000);
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (gsearch->search_results_tree_view), column);
 	
 //	gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (gsearch->search_results_tree_view), GTK_TREE_VIEW_GRID_LINES_VERTICAL);
@@ -1542,7 +1552,7 @@ populate_hit_counts (gpointer value,
 
 		if (type != -1) {
 			for (service = services; service->service; ++service) {
-				if (service->service_type == (guint32) type) {
+				if (strcmp(service->service,meta[0]) == 0) {
 					service->hit_count = atoi (meta[1]);
 					break;
 				}
@@ -1568,10 +1578,10 @@ update_page_count_label (GSearchWindow * gsearch)
 	
 	if (count > 5) {
 		/* Translators: this will appear like "Search results: 5 - 10 of 30 items" */
-		label_str = g_strdup_printf (_("%d - %d of %d hits"), from, to, count);
+		label_str = g_strdup_printf (_("%d - %d of %d items"), from, to, count);
 	} else 
 		/* Translators: this will appear like "Search results: 7 items" */
-		label_str = g_strdup_printf (ngettext ("%d item", "%d hits", count), count);
+		label_str = g_strdup_printf (ngettext ("%d item", "%d items", count), count);
 
 	gtk_label_set_text (GTK_LABEL (gsearch->count_label), label_str);
 	g_free (label_str);
@@ -1619,7 +1629,7 @@ get_hit_count (GPtrArray *out_array,
 	GSearchWindow *gsearch = user_data;
 
 	if (error) {
-		display_error_dialog (gsearch->window, error);
+		display_error_dialog (gsearch->window, _("Could not connect to search service as it may be busy"));
 		g_error_free (error);
 		return;
 	}
@@ -1784,6 +1794,25 @@ start_new_search (GSearchWindow * gsearch,
 	tracker_search_text_get_hit_count_all_async (tracker_client, query, (TrackerGPtrArrayReply) get_hit_count, gsearch);
 }
 
+
+void
+end_refresh_count (int count, GError * error, gpointer user_data)
+{
+	GSearchWindow *gsearch = user_data;
+	service_info_t	* service;
+
+	for (service = services; service->service; ++service) {
+		if (service->service_type == gsearch->current_service->service_type) {
+			service->hit_count = count;
+			break;
+		}
+	}
+
+	update_page_count_label (gsearch);
+
+}
+
+
 void
 end_search (GPtrArray * out_array,
 	    GError * error,
@@ -1795,16 +1824,39 @@ end_search (GPtrArray * out_array,
 	stop_animation (gsearch);
 	
 	if (error) {
-		display_error_dialog (gsearch->window, error);
+		display_error_dialog (gsearch->window,  _("Could not connect to search service as it may be busy"));
 		g_error_free (error);
 		return;
 	}
 
+	GError *error2 = NULL;
+	gchar* status = tracker_get_status (tracker_client, &error2);
+
+	if (error2) {
+		g_error_free (error2);
+		status = g_strdup ("Indexing");
+	}
+
+	if (strcmp (status, "Idle") == 0) {
+		gtk_widget_hide (gsearch->warning_label);
+	} else {
+		gtk_widget_show (gsearch->warning_label);
+	}
+	
+	g_free (status);
+
 	if (out_array) {
 
 		gsearch->current_service->has_hits = TRUE;
 
-		update_page_count_label (gsearch);
+		/* update hit count after search in case of dud hits */
+
+		tracker_search_text_get_hit_count_async	(tracker_client, gsearch->current_service->service_type, 
+					    		 gsearch->search_term,
+						    	 (TrackerIntReply)end_refresh_count,
+					    		 gsearch);
+
+		
 
 		gsearch->search_results_list_store = gsearch->current_service->store;
 
@@ -1860,7 +1912,7 @@ do_search (GSearchWindow * gsearch,
 			return;
 		}
 	}
-
+	
 	gsearch->current_service->offset = search_offset;
 	tracker_search_text_detailed_async (tracker_client,
 					    -1,
@@ -1897,6 +1949,7 @@ gsearch_app_create (GSearchWindow * gsearch)
 	g_hash_table_insert (gsearch->category_table, g_strdup ("Development"), &services[8]);
 	g_hash_table_insert (gsearch->category_table, g_strdup ("Conversations"), &services[9]);
 	g_hash_table_insert (gsearch->category_table, g_strdup ("Applications"), &services[10]);
+	g_hash_table_insert (gsearch->category_table, g_strdup ("WebHistory"), &services[11]);
 	//g_hash_table_insert (gsearch->category_table, g_strdup ("EmailAttachments"), &services[11]);
 	
 	gsearch->category_store = gtk_list_store_new (NUM_CATEGORY_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
@@ -1963,20 +2016,22 @@ gsearch_app_create (GSearchWindow * gsearch)
 
 
 	GtkWidget * widget;
+	char *search_label;
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (container), hbox, FALSE, FALSE, 3);
 
-	gsearch->name_and_folder_table = gtk_table_new (1, 4, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (gsearch->name_and_folder_table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (gsearch->name_and_folder_table), 12);
+	gsearch->name_and_folder_table = gtk_table_new (2, 4, FALSE);
 	gtk_container_add (GTK_CONTAINER (hbox), gsearch->name_and_folder_table);
 
-	label = gtk_label_new_with_mnemonic (_("_Search:"));
+	label = gtk_label_new (NULL);
+	search_label = g_strconcat ("<b>", _("_Search:"), "</b>", NULL);
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), search_label);
+	g_free (search_label);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
-
-	gtk_table_attach (GTK_TABLE (gsearch->name_and_folder_table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 1);
+	
+	gtk_table_attach (GTK_TABLE (gsearch->name_and_folder_table), label, 0, 1, 0, 1, GTK_FILL, 0, 6, 1);
 
 	gsearch->search_entry = sexy_icon_entry_new ();
 	sexy_icon_entry_add_clear_button (SEXY_ICON_ENTRY (gsearch->search_entry));
@@ -1984,8 +2039,14 @@ gsearch_app_create (GSearchWindow * gsearch)
 	entry =  (gsearch->search_entry);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
 
+	gsearch->warning_label = gtk_label_new (_("Tracker is still indexing so not all search results are available yet"));
+	gtk_label_set_selectable (GTK_LABEL (gsearch->warning_label), TRUE);
+	gtk_label_set_justify (GTK_LABEL (gsearch->warning_label), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (gsearch->warning_label), "xalign", 0.0, NULL);
+	gtk_table_attach (GTK_TABLE (gsearch->name_and_folder_table), gsearch->warning_label, 0, 2, 1, 2, GTK_FILL, 0, 6, 1);
+
 	hbox = gtk_hbutton_box_new ();
-	gtk_table_attach (GTK_TABLE (gsearch->name_and_folder_table), hbox, 3, 4, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_table_attach (GTK_TABLE (gsearch->name_and_folder_table), hbox, 3, 4, 0, 1, GTK_FILL, 0, 6, 0);
 
 	gsearch->find_button = gtk_button_new_from_stock (GTK_STOCK_FIND);
 	gtk_container_add (GTK_CONTAINER (hbox), gsearch->find_button);
@@ -2020,7 +2081,7 @@ gsearch_app_create (GSearchWindow * gsearch)
 	
 	gsearch->no_results = NULL;
 
-	gtk_paned_pack2 (GTK_PANED (gsearch->pane), vbox, TRUE, TRUE);		
+	gtk_paned_pack2 (GTK_PANED (gsearch->pane), vbox, TRUE, FALSE);
 
 	/* search results panel */
 
@@ -2045,7 +2106,7 @@ gsearch_app_create (GSearchWindow * gsearch)
 	/* category sidebar */
 
 	widget = create_sidebar (gsearch);
-	gtk_paned_pack1 (GTK_PANED (gsearch->pane), widget, TRUE, TRUE);
+	gtk_paned_pack1 (GTK_PANED (gsearch->pane), widget, TRUE, FALSE);
 
 	gtk_widget_set_sensitive (gsearch->category_list, FALSE);
 	
@@ -2060,6 +2121,8 @@ gsearch_app_create (GSearchWindow * gsearch)
 
 	gtk_widget_show_all (main_container);
 
+	gtk_widget_hide (gsearch->warning_label);
+
 	return gsearch->window;
 }
 
@@ -2151,9 +2214,9 @@ tracker_search_select_service_type_by_string (GtkComboBox * combo,
 gchar *
 tracker_search_pixmap_file (const gchar * partial_path)
 {
-	gchar * path;
+ 	gchar * path;
 
-	path = g_build_filename (TRACKER_DATADIR "/pixmaps/tracker", partial_path, NULL);
+	path = g_build_filename (TRACKER_DATADIR "/tracker/icons", partial_path, NULL);
 	if (g_file_test (path, G_FILE_TEST_EXISTS)) {
 		return path;
 	} else {
@@ -2207,7 +2270,7 @@ main (gint argc,
 
 	tracker_search_ui_manager (gsearch);
 
-	gtk_window_set_icon_name (GTK_WINDOW (gsearch->window), "gnome-searchtool");
+	gtk_window_set_icon_name (GTK_WINDOW (gsearch->window), "system-search");
 
 	gtk_window_set_default_icon_name ("tracker");
 
