@@ -1,7 +1,7 @@
 /* Tracker - indexer and metadata database engine
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  * Copyright (C) 2007, Jason Kivlighn (jkivlighn@gmail.com)
- * Copyright (C) 2007, Creative Commons (http://creativecommons.org)
+ * Copyright (C) 2007, Creative Commons (http://creativecommons.org) 
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -33,6 +33,10 @@
 extern Tracker *tracker;
 
 #define XMP_MIME_TYPE "application/rdf+xml"
+#define STACK_SIZE 30
+#define BLACK_LIST_SECONDS 3600
+#define MAX_DURATION 180
+#define MAX_CHANGE_TIMES 3
 
 typedef struct {
 	DBConnection	*db_con;
@@ -158,7 +162,7 @@ FileInfo *
 tracker_db_get_file_info (DBConnection *db_con, FileInfo *info)
 {
 	char *path, *name;
-	char *apath, *aname;
+//	char *apath, *aname;
 	char ***res;
 
 	g_return_val_if_fail (db_con, info);
@@ -171,14 +175,14 @@ tracker_db_get_file_info (DBConnection *db_con, FileInfo *info)
 	name = g_path_get_basename (info->uri);
 	path = g_path_get_dirname (info->uri);
 
-	apath = tracker_escape_string (path);
-	aname = tracker_escape_string (name);
-
-	g_free (aname);
-	g_free (apath);
+	//apath = tracker_escape_string (path);
+	//aname = tracker_escape_string (name);
 
 	res = tracker_exec_proc (db_con->index, "GetServiceID", 2, path, name);
 
+//	g_free (aname);
+//	g_free (apath);
+
 	g_free (name);
 	g_free (path);
 
@@ -434,7 +438,7 @@ tracker_db_get_pending_file (DBConnection *db_con, const char *uri)
 
 
 static void
-make_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const char *mime, int counter, TrackerChangeAction action, gboolean is_directory, gboolean is_new, int service_type_id)
+make_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const char *moved_to_uri, const char *mime, int counter, TrackerChangeAction action, gboolean is_directory, gboolean is_new, int service_type_id)
 {
 	char *str_file_id, *str_action, *str_counter;
 
@@ -445,9 +449,10 @@ make_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const
 	str_counter = tracker_int_to_str (counter);
 
 	if (tracker->is_running) {
-		if ( (counter > 0)
-		     || ((action == TRACKER_ACTION_EXTRACT_METADATA) && (g_async_queue_length (tracker->file_metadata_queue) > tracker->max_extract_queue_size))
-		     || ((action != TRACKER_ACTION_EXTRACT_METADATA) && (g_async_queue_length (tracker->file_process_queue) > tracker->max_process_queue_size)) ) {
+
+		gboolean move_file = (action == TRACKER_ACTION_FILE_MOVED_FROM || action == TRACKER_ACTION_DIRECTORY_MOVED_FROM);
+			
+		if (!move_file && ((counter > 0) || (g_async_queue_length (tracker->file_process_queue) > tracker->max_process_queue_size))) {
 
 			/* tracker_log ("************ counter for pending file %s is %d ***********", uri, counter); */
 			if (!mime) {
@@ -457,6 +462,7 @@ make_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const
 			}
 
 		} else {
+
 			FileInfo *info;
 
 			info = tracker_create_file_info (uri, action, 0, WATCH_OTHER);
@@ -470,6 +476,10 @@ make_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const
 				info->mime = g_strdup (mime);
 			}
 
+			if (action == TRACKER_ACTION_FILE_MOVED_FROM || action == TRACKER_ACTION_DIRECTORY_MOVED_FROM) {
+				info->moved_to_uri = g_strdup (moved_to_uri);
+			}
+
 			if (action != TRACKER_ACTION_EXTRACT_METADATA) {
 				g_async_queue_push (tracker->file_process_queue, info);
 			} else {
@@ -536,35 +546,191 @@ tracker_db_add_to_extract_queue (DBConnection *db_con, FileInfo *info)
 		g_async_queue_push (tracker->file_metadata_queue, info);
 
 	} else {
-		tracker_db_insert_pending_file (db_con, info->file_id, info->uri, info->mime, 0, TRACKER_ACTION_EXTRACT_METADATA, info->is_directory, info->is_new, info->service_type_id);
+		tracker_db_insert_pending_file (db_con, info->file_id, info->uri, NULL, info->mime, 0, TRACKER_ACTION_EXTRACT_METADATA, info->is_directory, info->is_new, info->service_type_id);
 	}
 
 	tracker_notify_meta_data_available ();
 }
 
+static void
+refresh_file_change_queue (gpointer data, gpointer user_data)
+{
+	FileChange *change = (FileChange*)data;
+	int *current = (int *)user_data;
+
+	if ((*current - change->first_change_time) > MAX_DURATION) {
+		g_queue_remove_all (tracker->file_change_queue, data);
+		free_file_change (&change);
+	}
+}
+
+static gint
+uri_comp (gconstpointer a, gconstpointer b)
+{
+	FileChange *change = (FileChange *)a;
+	char *valuea = change->uri;
+	char *valueb = (char *)b;
+
+	return strcmp (valuea, valueb);
+}
+
+static gint
+file_change_sort_comp (gconstpointer a, gconstpointer b, gpointer user_data)
+{
+	FileChange *changea, *changeb;
+	changea = (FileChange *)a;
+	changeb = (FileChange *)b;
+
+	if ((changea->num_of_change - changeb->num_of_change) == 0) {
+		return changea->first_change_time - changeb->first_change_time;
+	} else {
+		return changea->num_of_change - changeb->num_of_change;
+	}
+}
+
+static void
+print_file_change_queue ()
+{
+	GList *head, *l;
+	FileChange *change;
+	gint count;
+
+	head = g_queue_peek_head_link (tracker->file_change_queue);
+
+	tracker_log ("File Change queue is:");
+	count = 1;
+	for (l = g_list_first (head); l != NULL; l = g_list_next (l)) {
+		change = (FileChange*)l->data;
+		tracker_info ("%d\t%s\t%d\t%d",
+			 count++, change->uri,
+			 change->first_change_time,
+			 change->num_of_change);
+	}
+	
+}
+
+static void
+index_blacklist_file (char *uri)
+{
+	FileInfo *info;
+
+	info = tracker_create_file_info (uri, TRACKER_ACTION_FILE_CHECK, 0, WATCH_OTHER);
+
+	info->is_directory = FALSE;
+	
+	info->is_new = FALSE;
+
+	info->mime = g_strdup ("unknown");
+
+	g_async_queue_push (tracker->file_process_queue, info);
+	
+	tracker_notify_file_data_available ();
+
+}
+
+
+static gboolean
+index_black_list ()
+{
+
+	g_slist_foreach (tracker->tmp_black_list, (GFunc) index_blacklist_file, NULL);
+	
+	g_slist_foreach (tracker->tmp_black_list, (GFunc) g_free, NULL);
+	
+	g_slist_free (tracker->tmp_black_list);
+	
+	tracker->tmp_black_list = NULL;
+	
+	tracker->black_list_timer_active = FALSE;
+	
+	return FALSE;
+
+}
+
+
+
+static gboolean
+check_uri_changed_frequently (const char *uri)
+{
+	GList *find;
+	FileChange *change;
+	time_t current;
+
+	if (!tracker->file_change_queue) {
+		/* init queue */
+		tracker->file_change_queue = g_queue_new ();
+	}
+
+	current = time (NULL);
+
+	/* remove items which are very old */
+	g_queue_foreach (tracker->file_change_queue,
+			 refresh_file_change_queue, &current);
+
+	find = g_queue_find_custom (tracker->file_change_queue, uri, uri_comp);
+	if (!find) {
+		/* not found, add to in the queue */
+				
+		change = g_new0 (FileChange, 1);
+		change->uri = g_strdup (uri);
+		change->first_change_time = current;
+		change->num_of_change = 1;
+		if (g_queue_get_length (tracker->file_change_queue) == STACK_SIZE) {
+			FileChange *tmp = (FileChange*) g_queue_pop_head (
+						tracker->file_change_queue);
+			free_file_change (&tmp);
+		}
+		g_queue_insert_sorted (tracker->file_change_queue, change,
+					file_change_sort_comp, NULL);
+		print_file_change_queue ();
+		return FALSE;
+	} else {
+		change = (FileChange *) find->data;
+		(change->num_of_change)++;
+		g_queue_sort (tracker->file_change_queue,
+			file_change_sort_comp, NULL);
+		if (change->num_of_change < MAX_CHANGE_TIMES) {
+			print_file_change_queue ();
+			return FALSE;
+		} else {
+			print_file_change_queue ();
+			
+			/* add uri to blacklist */
+			
+			tracker_log ("blacklisting %s", change->uri);
+			
+			tracker->tmp_black_list = g_slist_prepend (tracker->tmp_black_list, g_strdup (change->uri));
+			
+			if (!tracker->black_list_timer_active) {
+				tracker->black_list_timer_active = TRUE;
+				g_timeout_add_seconds (BLACK_LIST_SECONDS, (GSourceFunc) index_black_list, NULL);
+			}
+			
+			g_queue_remove_all (tracker->file_change_queue, change);
+			free_file_change (&change);
+			
+			return TRUE;
+		}
+	}
+
+}
 
 void
-tracker_db_insert_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const char *mime, int counter, TrackerChangeAction action, gboolean is_directory, gboolean is_new, int service_type_id)
+tracker_db_insert_pending_file (DBConnection *db_con, guint32 file_id, const char *uri, const char *moved_to_uri, const char *mime, int counter, TrackerChangeAction action, gboolean is_directory, gboolean is_new, int service_type_id)
 {
 	FileInfo *info;
 
 	g_return_if_fail (tracker_check_uri (uri));
 
-	/* if a check action then then discard if up to date
-	if (action == TRACKER_ACTION_CHECK || action == TRACKER_ACTION_FILE_CHECK || action == TRACKER_ACTION_DIRECTORY_CHECK) {
-
-		guint32 id;
-
-		if (tracker_db_is_file_up_to_date (db_con->index, uri, &id)) {
-			return;
-		}
-
-		if (file_id == 0) {
-			file_id = id;
-		}
-
+	/* check if uri changed too frequently */
+	if (((action == TRACKER_ACTION_CHECK) ||
+		(action == TRACKER_ACTION_FILE_CHECK) || (action == TRACKER_ACTION_WRITABLE_FILE_CLOSED)) &&
+		check_uri_changed_frequently (uri)) {
+		
+		return;
 	}
-*/
+	
+	
 	/* check if uri already has a pending action and update accordingly */
 	info = tracker_db_get_pending_file (db_con, uri);
 
@@ -622,7 +788,7 @@ tracker_db_insert_pending_file (DBConnection *db_con, guint32 file_id, const cha
 		tracker_free_file_info (info);
 
 	} else {
-		make_pending_file (db_con, file_id, uri, mime, counter, action, is_directory, is_new, service_type_id);
+		make_pending_file (db_con, file_id, uri, moved_to_uri, mime, counter, action, is_directory, is_new, service_type_id);
 	}
 }
 
@@ -691,8 +857,8 @@ tracker_db_index_service (DBConnection *db_con, FileInfo *info, const char *serv
 	}
 
 	if (info->mime == NULL) {
-                info->mime = g_strdup("unknown");
-        }
+		info->mime = g_strdup("unknown");
+	}
 
 	if (info->is_new) {
 		if (info->mime)
@@ -927,9 +1093,9 @@ tracker_db_index_master_files (DBConnection *db_con, const gchar *dirname, const
 void
 tracker_db_index_file (DBConnection *db_con, FileInfo *info, const char *attachment_uri, const char *attachment_service)
 {
-	char *services_with_metadata[] = {"Documents", "Music", "Videos", "Images", NULL};
-	char *services_with_text[] = {"Documents", "Development", "Text", NULL};
-	char *services_with_thumbs[] = {"Documents", "Images", "Videos", NULL};
+	char *services_with_metadata[] = {"Documents", "Music", "Videos", "Images","WebHistory", NULL};
+	char *services_with_text[] = {"Documents", "Development", "Text", "WebHistory",NULL};
+	char *services_with_thumbs[] = {"Documents", "Images", "Videos", "WebHistory",NULL};
 
 	GHashTable	*meta_table;
 	const char	*ext;
@@ -1002,6 +1168,52 @@ tracker_db_index_file (DBConnection *db_con, FileInfo *info, const char *attachm
 		tracker_add_metadata_to_table  (meta_table, g_strdup ("File:Modified"), tracker_date_to_str (info->mtime));
 		tracker_add_metadata_to_table  (meta_table, g_strdup ("File:Accessed"), tracker_date_to_str (info->atime));
 
+                /* need to add special data for web history */
+                if (attachment_service != NULL && strcmp(attachment_service,"WebHistory") == 0)  {
+                	
+                	gchar* meta_file = g_strconcat(dirname,"/.",filename,NULL);
+                     
+                     	FILE* fp = g_fopen(meta_file, "r");
+                     
+                     	if (fp != NULL) {
+                          	char buf[512];
+                          	
+                          	fgets(buf,512,fp);  //get the first line, it is URL for this web history object
+                          	tracker_debug("URL for this WebHistory is %s\n",buf);
+                          	tracker_add_metadata_to_table  (meta_table, g_strdup ("Doc:URL"), g_strdup(buf));
+                          	
+                          	fgets(buf,512,fp);
+                          	fgets(buf,512,fp);
+                          	fgets(buf,512,fp);
+                          	fgets(buf,512,fp);  // get the keywords for this file
+                          	
+                          	if (buf != NULL) {
+                              		
+                              		/* format like t:dc:keyword=xxx */
+                              		gchar** keys = g_strsplit (buf,"=",0);
+                              
+                              		if (keys != NULL && strcmp(keys[0],"t:dc:keyword") == 0 && keys[1]) {
+
+                                		char *doc_keyword = g_strdup (keys[1]);
+
+	                        		tracker_debug("found keywords : %s\n",doc_keyword);
+	                        	
+                                  		tracker_add_metadata_to_table  (meta_table, g_strdup ("Doc:Keywords"), doc_keyword);
+                                	}
+                                
+                                
+                                	if (keys) g_strfreev(keys);
+                                
+                        	      	
+                      		}
+
+                        	fclose (fp);
+               		}
+                     	g_free (meta_file);
+                }
+                                
+
+
 		is_external_service = g_str_has_prefix (info->mime, "service/");
 		is_file_indexable = (!info->is_directory && (strcmp (info->mime, "unknown") != 0) && (strcmp (info->mime, "symlink") != 0) && tracker_file_is_indexable (info->uri));
 
@@ -1070,5 +1282,9 @@ tracker_db_index_conversation (DBConnection *db_con, FileInfo *info)
 	tracker_db_index_file (db_con, info, NULL, "GaimConversations");
 }
 
-
+void 
+tracker_db_index_webhistory(DBConnection *db_con, FileInfo *info)
+{
+	tracker_db_index_file (db_con, info, NULL, "WebHistory");
+}
 
