@@ -110,6 +110,8 @@ tracker_dbus_method_metadata_set (DBusRec *rec)
 	}
 
 	g_free (id);
+	dbus_free_string_array (keys);
+	dbus_free_string_array (values);
 
 	reply = dbus_message_new_method_return (rec->message);
 
@@ -179,6 +181,11 @@ tracker_dbus_method_metadata_get (DBusRec *rec)
 
 
 	res_service = tracker_db_get_service_for_entity (db_con, id);
+	
+	if (!res_service) {
+		tracker_set_error (rec, "Service info cannot be found for entity %s", uri);
+		return;
+	}
 
 	/* build SELECT clause */
 	sql = g_string_new (" SELECT DISTINCT ");
@@ -205,14 +212,17 @@ tracker_dbus_method_metadata_get (DBusRec *rec)
 		if (field->needs_join) {
 			g_string_append_printf (sql_join, "\n LEFT OUTER JOIN %s %s ON (S.ID = %s.ServiceID and %s.MetaDataID = %s) ", field->table_name, field->alias, field->alias, field->alias, field->id_field);
 		}
-			
 
+		tracker_free_metadata_field (field);
 	}
 
 	g_string_append (sql, sql_join->str);
 
 	g_string_free (sql_join, TRUE);
 
+	dbus_free_string_array (keys);
+	g_free (res_service);
+
 	/* build WHERE clause */
 
 	g_string_append_printf (sql, " WHERE S.ID = %s", id );
