@@ -22,7 +22,8 @@
 #define _TRACKER_DBUS_SEARCH_H_
 
 #include "tracker-dbus.h"
-
+void    tracker_dbus_method_search_get_hit_count		(DBusRec *rec);
+void	tracker_dbus_method_search_get_hit_count_all		(DBusRec *rec);
 void	tracker_dbus_method_search_text 			(DBusRec *rec);
 void	tracker_dbus_method_search_text_detailed		(DBusRec *rec);
 void	tracker_dbus_method_search_get_snippet 			(DBusRec *rec);
@@ -30,5 +31,6 @@ void	tracker_dbus_method_search_files_by_text 		(DBusRec *rec);
 void	tracker_dbus_method_search_metadata 			(DBusRec *rec);
 void	tracker_dbus_method_search_matching_fields 		(DBusRec *rec);
 void	tracker_dbus_method_search_query 			(DBusRec *rec);
+void	tracker_dbus_method_search_suggest 			(DBusRec *rec);
 
 #endif
