@@ -27,14 +27,7 @@
 #include "tracker-email-kmail.h"
 #include "tracker-email-utils.h"
 #include "tracker-db-email.h"
-
-#ifdef HAVE_INOTIFY
-#   include "tracker-inotify.h"
-#else
-#   ifdef HAVE_FAM
-#      include "tracker-fam.h"
-#   endif
-#endif
+#include "tracker-watch.h"
 
 
 typedef struct {
@@ -700,7 +693,6 @@ static gboolean
 index_mail_file_in_maildir_dir (DBConnection *db_con, const gchar *dir, FileInfo *info, MailType mail_type)
 {
         MailStore *store;
-        guint32   dummyId;
 
         if (tracker_db_email_get_mbox_id (db_con, dir) == -1) {
                 gchar *filename = g_path_get_basename (info->uri);
@@ -720,7 +712,7 @@ index_mail_file_in_maildir_dir (DBConnection *db_con, const gchar *dir, FileInfo
 
         tracker_log ("Looking for email file \"%s\"", info->uri);
 
-        if (!tracker_db_email_is_up_to_date (db_con, info->uri, &dummyId)) {
+        if (!tracker_db_email_is_saved_email_file (db_con, info->uri)) {
                 MailMessage *mail_msg = email_parse_mail_message_by_path (MAIL_APP_KMAIL, info->uri, NULL, NULL);
 
                 if (!mail_msg) {
@@ -740,8 +732,13 @@ index_mail_file_in_maildir_dir (DBConnection *db_con, const gchar *dir, FileInfo
 
                 email_free_mail_file (mail_msg->parent_mail_file);
                 email_free_mail_message (mail_msg);
-        } else {
 
+        } else {
+                /* A file is never touched at all (ever when a user read it) so its containt is never updated.
+                   When a email file is put in trash it is moved to the trash directory and it is given the
+                   suffix ":2,S"... For us it just means that trackerd will see a file being deleted and it
+                   will remove it from its database.
+                */
                 tracker_log ("Nothing need to be done, email file \"%s\" is already up to date", info->uri) ;
         }
 
