@@ -31,18 +31,12 @@
 
 #include "tracker-cache.h"
 #include "tracker-db-email.h"
+#include "tracker-dbus.h"
 #include "tracker-email-utils.h"
 #include "tracker-email-evolution.h"
 #include "tracker-email-thunderbird.h"
 #include "tracker-email-kmail.h"
-
-#ifdef HAVE_INOTIFY
-#   include "tracker-inotify.h"
-#else
-#   ifdef HAVE_FAM
-#      include "tracker-fam.h"
-#   endif
-#endif
+#include "tracker-watch.h"
 
 
 extern Tracker *tracker;
@@ -116,8 +110,9 @@ email_parse_mail_file_and_save_new_emails (DBConnection *db_con, MailApplication
 	MailMessage *mail_msg;
 	gint        indexed = 0, junk = 0, deleted = 0;
 
-
-	if (!tracker->is_running) return FALSE; 
+	if (!tracker->is_running) {
+                return FALSE;
+        }
 
 	g_return_val_if_fail (db_con, FALSE);
 	g_return_val_if_fail (path, FALSE);
@@ -125,6 +120,9 @@ email_parse_mail_file_and_save_new_emails (DBConnection *db_con, MailApplication
 
 	mf = email_open_mail_file_at_offset (mail_app, path, store->offset, TRUE);
 
+	tracker->mbox_count++;
+	tracker_dbus_send_index_progress_signal ("Emails", path);
+
 	while ((mail_msg = email_mail_file_parse_next (mf, read_mail_helper, read_mail_user_data))) {
 
 		if (!tracker->is_running) {
@@ -161,21 +159,10 @@ email_parse_mail_file_and_save_new_emails (DBConnection *db_con, MailApplication
 
 		email_free_mail_message (mail_msg);
 
-		LoopEvent event = tracker_cache_event_check (db_con->data, TRUE);
-
-		if (event==EVENT_SHUTDOWN || event==EVENT_DISABLE) {
-
-			tracker_db_end_index_transaction (db_con->data);
-			tracker_cache_flush_all (FALSE);
-
-			break;						
-
-		} else if (event == EVENT_CACHE_FLUSHED) {
-			
-			tracker_db_end_index_transaction (db_con->data);
-			tracker_db_start_index_transaction (db_con->data);		
-
-		}				
+		if (!tracker_cache_process_events (db_con->data, TRUE) ) {
+			tracker->shutdown = TRUE;
+			return FALSE;	
+		}
 
 		if (tracker_db_regulate_transactions (db_con->data, 500)) {
 
@@ -186,19 +173,18 @@ email_parse_mail_file_and_save_new_emails (DBConnection *db_con, MailApplication
 			if (tracker->index_count % 2500 == 0) {
 				tracker_db_end_index_transaction (db_con->data);
 				tracker_db_refresh_all (db_con->data);
-//				tracker_db_refresh_email (db_con);
 				tracker_db_start_index_transaction (db_con->data);
 			}
-
+			tracker_dbus_send_index_progress_signal ("Emails", path);			
 			
 		}	
 
-		
-
-		
+	
 	}
 
 	email_free_mail_file (mf);
+	tracker->mbox_processed++;
+	tracker_dbus_send_index_progress_signal ("Emails", path);
 
 	if (indexed > 0) {
 		tracker_info ("Indexed %d emails in email store %s and ignored %d junk and %d deleted emails",
@@ -741,32 +727,6 @@ email_get_mime_infos_from_mime_file (const gchar *mime_file)
 }
 
 
-void
-email_index_each_email_attachment (DBConnection *db_con, const MailMessage *mail_msg)
-{
-	const GSList *tmp;
-
-        g_return_if_fail (db_con);
-	g_return_if_fail (mail_msg);
-
-	return;
-
-	for (tmp = mail_msg->attachments; tmp; tmp = tmp->next) {
-		const MailAttachment	*ma;
-		FileInfo		*info;
-
-		ma = tmp->data;
-
-		info = tracker_create_file_info (ma->tmp_decoded_file, TRACKER_ACTION_CHECK, 0, WATCH_OTHER);
-		info->is_directory = FALSE;
-		info->mime = g_strdup (ma->mime);
-
-		g_async_queue_push (tracker->file_process_queue, info);
-		tracker_notify_file_data_available ();
-	}
-}
-
-
 gboolean
 email_add_saved_mail_attachment_to_mail_message (MailMessage *mail_msg, MailAttachment *ma)
 {
@@ -854,9 +814,6 @@ email_decode_mail_attachment_to_file (const gchar *src, const gchar *dst, MimeEn
 	g_mime_stream_write_to_stream (filtered_stream, stream_dst);
 	g_mime_stream_flush (filtered_stream);
 
-	//g_mime_stream_close (filtered_stream);
-	//g_mime_stream_close (stream_dst);
-
 	g_object_unref (filtered_stream);
 	g_object_unref (stream_src);
 	g_object_unref (stream_dst);
