@@ -101,9 +101,11 @@ void		tracker_index_free_hit_list		(GSList *hit_list);
 
 Indexer * 	tracker_indexer_open 			(const gchar *name);
 void		tracker_indexer_close 			(Indexer *indexer);
+gboolean	tracker_indexer_repair 			(const char *name);
 void		tracker_indexer_free 			(Indexer *indexer, gboolean remove_file);
 gboolean	tracker_indexer_has_merge_index 	(Indexer *indexer, gboolean update);
 
+
 guint32		tracker_indexer_size 			(Indexer *indexer);
 gboolean	tracker_indexer_optimize		(Indexer *indexer);
 void		tracker_indexer_sync 			(Indexer *indexer);
@@ -111,6 +113,7 @@ void		tracker_indexer_sync 			(Indexer *indexer);
 void		tracker_indexer_apply_changes 		(Indexer *dest, Indexer *src,  gboolean update);
 void		tracker_indexer_merge_indexes 		(IndexType type);
 gboolean	tracker_indexer_has_merge_files 	(IndexType type);
+gboolean	tracker_indexer_has_tmp_merge_files 	(IndexType type);
 
 /* Indexing api */
 gboolean	tracker_indexer_append_word 		(Indexer *indexer, const gchar *word, guint32 id, gint service, gint score);
@@ -126,7 +129,7 @@ gboolean	tracker_indexer_get_hits 		(SearchQuery *query);
 gchar ***	tracker_get_hit_counts 			(SearchQuery *query);
 gint		tracker_get_hit_count 			(SearchQuery *query);
 
-gchar ***	tracker_get_words_starting_with 	(Indexer *indexer, const gchar *word);
+gboolean	tracker_remove_dud_hits 		(Indexer *indexer, const gchar *word, GSList *dud_list);
 
 
 #endif
