@@ -40,7 +40,7 @@ GHashTable *	tracker_parse_text (GHashTable *word_table, const char *txt, int we
 
 GHashTable *	tracker_parse_text_fast (GHashTable *word_table, const char *txt, int weight);
 
-char *		tracker_parse_text_to_string (const char *txt, gboolean filter_words, gboolean delimit);	
+char *		tracker_parse_text_to_string (const char *txt, gboolean filter_words, gboolean filter_numbers, gboolean delimit);	
 
 char **		tracker_parse_text_into_array (const char *text);
 
