@@ -134,19 +134,6 @@ typedef enum {
 } LogicOperators;
 
 
-typedef struct {
-	char 		*alias;
-	char 	 	*field_name;
-	char	 	*meta_field;
-	char	 	*table_name;
-	char	 	*id_field;
-	DataTypes	data_type;
-	gboolean	multiple_values;
-	gboolean 	is_select;
-	gboolean 	is_condition;
-
-} FieldData;
-
 
 typedef struct {
 	GMarkupParseContext 	*context;
@@ -345,34 +332,6 @@ pop_stack_until (ParserData *data, ParseState state)
 }
 
 
-static void
-free_metadata_field (FieldData *field_data)
-{
-	g_return_if_fail (field_data);
-
-	if (field_data->alias) {
-		g_free (field_data->alias);
-	}
-
-	if (field_data->field_name) {
-		g_free (field_data->field_name);
-	}
-
-	if (field_data->meta_field) {
-		g_free (field_data->meta_field);
-	}
-
-	if (field_data->table_name) {
-		g_free (field_data->table_name);
-	}
-
-	if (field_data->id_field) {
-		g_free (field_data->id_field);
-	}
-
-	g_free (field_data);
-}
-
 
 static FieldData *
 add_metadata_field (ParserData *data, const char *field_name, gboolean is_select, gboolean is_condition)
@@ -391,25 +350,27 @@ add_metadata_field (ParserData *data, const char *field_name, gboolean is_select
 		tmp_field = tmp->data;
 
 		if (tmp_field && tmp_field->field_name) {
-			if (strcmp (tmp_field->field_name, field_name) == 0) {
+			if (strcasecmp (tmp_field->field_name, field_name) == 0) {
 
 				field_data = tmp_field;
 		
-				if ((field_data->multiple_values && field_data->is_condition && is_select) || (field_data->multiple_values && field_data->is_select && is_condition) ) {
-					field_exists = FALSE;
-				} else {
-					field_exists = TRUE;
+				field_exists = TRUE;
 	
-					if (is_condition) {
-						field_data->is_condition = TRUE;
-					} 
+				if (is_condition) {
+					field_data->is_condition = TRUE;
+				} 
 
-					if (is_select) {
+				if (is_select) {
+					if (!field_data->is_select) {
+				
 						field_data->is_select = TRUE;
+						g_string_append_printf (data->sql_select, ", %s", field_data->select_field);
 					}
-
-					break;
+					
 				}
+
+				break;
+				
 			}
 		}
 	}
@@ -417,66 +378,17 @@ add_metadata_field (ParserData *data, const char *field_name, gboolean is_select
 	
 
 	if (!field_exists) {
-		FieldDef *def;
-		char	 *istr;
-		int	 i;
 
-		field_data = g_new0 (FieldData, 1);
-
-		field_data->is_select = is_select;
-		field_data->is_condition = is_condition;
-		field_data->field_name = g_strdup (field_name);
-
-		i = g_slist_length (data->fields);
-
-		istr = tracker_int_to_str (i);
-
-		field_data->alias = g_strconcat ("M", istr, NULL);
-		g_free (istr);
-
-		def = tracker_db_get_field_def (data->db_con, field_name);
-
-		if (def) {
-			char *st;
-			
-			if (is_select && def->multiple_values) {
-				st = g_strdup ("ServiceMetaDataDisplay");
-			} else {
-				st = tracker_get_metadata_table (def->type);
-			}
-
-			field_data->data_type = def->type;
-			field_data->meta_field = g_strconcat (field_data->alias, ".MetaDataValue", NULL);
-			field_data->table_name = g_strdup (st);
-			field_data->id_field = g_strdup (def->id);
-			field_data->multiple_values = def->multiple_values;
-			
+		field_data = tracker_db_get_metadata_field (data->db_con, data->service, field_name, g_slist_length (data->fields), is_select, is_condition);
+		if (field_data) {
 			data->fields = g_slist_prepend (data->fields, field_data);
-
 			if (is_select) {
-
-			/* leave datetime fields as integers (seconds from epoch) so clients can format as they wish */
-
-//				if (def->type == DATA_DATE) {
-//					g_string_append_printf (data->sql_select, ", FormatDate(%s)", field_data->meta_field);
-//				} else {
-				g_string_append_printf (data->sql_select, ", %s", field_data->meta_field);
-//				}
-
+				g_string_append_printf (data->sql_select, ", %s", field_data->select_field);
 			}
 
-			g_free (st);
-
-			tracker_db_free_field_def (def);
-
-		} else {
-			g_free (field_data);
-			return NULL;
-		}
-
+		} 
 	} 
 	
-
 	return field_data;
 }
 
@@ -809,12 +721,12 @@ build_sql (ParserData *data)
 
 			sub = strchr (data->current_value, '*');
 			if (sub) {
-				g_string_append_printf (str, " (%s glob '%s') ", field_data->meta_field, data->current_value);
+				g_string_append_printf (str, " (%s glob '%s') ", field_data->where_field, data->current_value);
 			} else {
-				if (field_data->data_type == DATA_DATE || field_data->data_type == DATA_NUMERIC) {
-					g_string_append_printf (str, " (%s = %s) ", field_data->meta_field, value);
+				if (field_data->data_type == DATA_DATE || field_data->data_type == DATA_INTEGER || field_data->data_type == DATA_DOUBLE) {
+					g_string_append_printf (str, " (%s = %s) ", field_data->where_field, value);
 				} else {
-					g_string_append_printf (str, " (%s = '%s') ", field_data->meta_field, value);
+					g_string_append_printf (str, " (%s = '%s') ", field_data->where_field, value);
 				}
 			}
 
@@ -822,25 +734,25 @@ build_sql (ParserData *data)
 
 		case OP_GREATER:
 
-			g_string_append_printf (str, " (%s > %s) ", field_data->meta_field, value);
+			g_string_append_printf (str, " (%s > %s) ", field_data->where_field, value);
 
 			break;
 
 		case OP_GREATER_EQUAL:
 
-			g_string_append_printf (str, " (%s >= %s) ", field_data->meta_field, value);
+			g_string_append_printf (str, " (%s >= %s) ", field_data->where_field, value);
 
 			break;
 
 		case OP_LESS:
 
-			g_string_append_printf (str, " (%s < %s) ", field_data->meta_field, value);
+			g_string_append_printf (str, " (%s < %s) ", field_data->where_field, value);
 
 			break;
 
 		case OP_LESS_EQUAL:
 
-			g_string_append_printf (str, " (%s <= %s) ", field_data->meta_field, value);
+			g_string_append_printf (str, " (%s <= %s) ", field_data->where_field, value);
 
 			break;
 
@@ -849,9 +761,9 @@ build_sql (ParserData *data)
 			sub = strchr (data->current_value, '*');
 
 			if (sub) {
-				g_string_append_printf (str, " (%s like '%s%s%s') ", field_data->meta_field, "%", data->current_value, "%");
+				g_string_append_printf (str, " (%s like '%s%s%s') ", field_data->where_field, "%", data->current_value, "%");
 			} else {
-				g_string_append_printf (str, " (%s like '%s%s%s') ", field_data->meta_field, "%", data->current_value, "%");
+				g_string_append_printf (str, " (%s like '%s%s%s') ", field_data->where_field, "%", data->current_value, "%");
 			}
 
 			break;
@@ -861,16 +773,16 @@ build_sql (ParserData *data)
 			sub = strchr (data->current_value, '*');
 
 			if (sub) {
-				g_string_append_printf (str, " (%s like '%s') ", field_data->meta_field, data->current_value);
+				g_string_append_printf (str, " (%s like '%s') ", field_data->where_field, data->current_value);
 			} else {
-				g_string_append_printf (str, " (%s like '%s%s') ", field_data->meta_field, data->current_value, "%");
+				g_string_append_printf (str, " (%s like '%s%s') ", field_data->where_field, data->current_value, "%");
 			}
 			
 			break;
 
 		case OP_REGEX:
 
-			g_string_append_printf (str, " (%s REGEXP '%s') ", field_data->meta_field, data->current_value);
+			g_string_append_printf (str, " (%s REGEXP '%s') ", field_data->where_field, data->current_value);
 
 			break;
 
@@ -880,7 +792,7 @@ build_sql (ParserData *data)
 			
 			if (s && s[0]) {
 
-				g_string_append_printf (str, " (%s in ('%s'", field_data->meta_field, s[0]);
+				g_string_append_printf (str, " (%s in ('%s'", field_data->where_field, s[0]);
 
 				char **p;
 				for (p = s+1; *p; p++) {
@@ -1099,9 +1011,10 @@ error_handler (GMarkupParseContext *context,
 	       GError		   *error,
 	       gpointer		   user_data)
 {
-	tracker_log ("Error in rdf query parse: %s", error->message);
+	tracker_error ("ERROR: in rdf query parse: %s", error->message);
 }
 
+
 static GString *
 get_select_header (const char *service) 
 {
@@ -1153,6 +1066,7 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 	memset (&data, 0, sizeof (data));
 	data.db_con = db_con;
 	data.statement_count = 0;
+	data.service = (char *) service;
 
 	data.sql_select = get_select_header (service);
 
@@ -1165,8 +1079,8 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 			field_data = add_metadata_field (&data, fields[i], TRUE, FALSE);
 
 			if (!field_data) {
-				tracker_log ("RDF Query failed : field %s not found", fields[i]);
-				g_slist_foreach (data.fields, (GFunc) free_metadata_field, NULL);
+				tracker_error ("ERROR: RDF Query failed: field %s not found", fields[i]);
+				g_slist_foreach (data.fields, (GFunc) tracker_free_metadata_field, NULL);
 				g_slist_free (data.fields);
 				g_string_free (data.sql_select, TRUE);
 				return NULL;
@@ -1184,14 +1098,14 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 	tracker_debug ("search term is %s", search_text);
 
 
-	if (search_text && (strlen (search_text) > 0)) {
+	if (!tracker_is_empty_string (search_text)) {
 		do_search = TRUE;
 		g_string_append_printf (data.sql_from, "\n FROM %s S INNER JOIN SearchResults1 M ON S.ID = M.SID ", table_name);
 	} else {
 		g_string_append_printf (data.sql_from, "\n FROM %s S ", table_name);
 	}
 
-	if (keyword && strlen (keyword) > 0) {
+	if (!tracker_is_empty_string (keyword)) {
 		char *keyword_metadata = tracker_get_related_metadata_names (db_con, "DC:Keywords");
 		g_string_append_printf (data.sql_from, "\n INNER JOIN ServiceKeywordMetaData K ON S.ID = K.ServiceID and K.MetaDataID in (%s) and K.MetaDataValue = '%s' ", keyword_metadata, keyword);
 		g_free (keyword_metadata);
@@ -1200,9 +1114,9 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 	data.sql_where = g_string_new ("");
 
 	if (strlen (query) < 10) {
-		g_string_append_printf (data.sql_where, "\n WHERE (S.ServiceTypeID between GetServiceTypeID('%s') and GetMaxServiceTypeID('%s')) ", service, service);
+		g_string_append_printf (data.sql_where, "\n WHERE (S.ServiceTypeID in (select TypeId from ServiceTypes where TypeName = '%s' or Parent = '%s')) ", service, service);
 	} else {
-		g_string_append_printf (data.sql_where, "\n WHERE (S.ServiceTypeID between GetServiceTypeID('%s') and GetMaxServiceTypeID('%s')) AND ", service, service);
+		g_string_append_printf (data.sql_where, "\n WHERE (S.ServiceTypeID in (select TypeId from ServiceTypes where TypeName = '%s' or Parent = '%s')) AND ", service, service);
 	}
 
 	if (limit < 1) {
@@ -1258,7 +1172,9 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 			tmp_field = tmp->data;
 
 			if (!tmp_field->is_condition) {
-				g_string_append_printf (data.sql_from, "\n LEFT OUTER JOIN %s %s ON (S.ID = %s.ServiceID and %s.MetaDataID = %s) ", tmp_field->table_name, tmp_field->alias, tmp_field->alias, tmp_field->alias, tmp_field->id_field);
+				if (tmp_field->needs_join) {
+					g_string_append_printf (data.sql_from, "\n LEFT OUTER JOIN %s %s ON (S.ID = %s.ServiceID and %s.MetaDataID = %s) ", tmp_field->table_name, tmp_field->alias, tmp_field->alias, tmp_field->alias, tmp_field->id_field);
+				}
 			} else {
 				char *related_metadata = tracker_get_related_metadata_names (db_con, tmp_field->field_name);
 				g_string_append_printf (data.sql_from, "\n INNER JOIN %s %s ON (S.ID = %s.ServiceID and %s.MetaDataID in (%s)) ", tmp_field->table_name, tmp_field->alias, tmp_field->alias, tmp_field->alias, related_metadata);
@@ -1274,7 +1190,7 @@ tracker_rdf_query_to_sql (DBConnection *db_con, const char *query, const char *s
 		g_string_free (data.sql_order, TRUE);
 	}
 
-	g_slist_foreach (data.fields, (GFunc) free_metadata_field, NULL);
+	g_slist_foreach (data.fields, (GFunc) tracker_free_metadata_field, NULL);
 	g_slist_free (data.fields);
 
 	g_slist_free (data.stack);
