@@ -48,7 +48,7 @@ extern char *tracker_actions[];
 
 #define TRACKER_DB_VERSION_REQUIRED	13
 #define TRACKER_VERSION			VERSION
-#define TRACKER_VERSION_INT		603
+#define TRACKER_VERSION_INT		604
 
 /* default performance options */
 #define MAX_INDEX_TEXT_LENGTH		1048576
@@ -124,6 +124,7 @@ typedef enum {
 	INDEX_CONFIG,
 	INDEX_APPLICATIONS,
 	INDEX_FILES,
+	INDEX_WEBHISTORY,
 	INDEX_CRAWL_FILES,
 	INDEX_CONVERSATIONS,	
 	INDEX_EXTERNAL,	
@@ -137,11 +138,11 @@ typedef struct {
 	DataTypes	type;
 	char 		*field_name;
 	int		weight;
-	gboolean	embedded;
-	gboolean	multiple_values;
-	gboolean	delimited;
-	gboolean	filtered;
-	gboolean	store_metadata;
+	guint           embedded : 1;
+	guint           multiple_values : 1;
+	guint           delimited : 1;
+	guint           filtered : 1;
+	guint           store_metadata : 1;
 
 	GSList		*child_ids; /* related child metadata ids */
 
@@ -156,10 +157,10 @@ typedef struct {
 	char	 	*table_name;
 	char	 	*id_field;
 	DataTypes	data_type;
-	gboolean	multiple_values;
-	gboolean 	is_select;
-	gboolean 	is_condition;
-	gboolean	needs_join;
+	guint           multiple_values : 1;
+	guint           is_select : 1;
+	guint           is_condition : 1;
+	guint           needs_join : 1;
 
 } FieldData;
 
@@ -169,16 +170,17 @@ typedef struct {
 	int		id;
 	char 		*name;
 	char		*parent;
-	gboolean	enabled;
-	gboolean	embedded;
-	gboolean	has_metadata;
-	gboolean	has_fulltext;
-	gboolean	has_thumbs;
 	char		*content_metadata;
 	GSList		*key_metadata;
 	DBTypes		database;
-	gboolean 	show_service_files;
-	gboolean 	show_service_directories;
+
+	guint           enabled : 1;
+	guint           embedded : 1;
+	guint           has_metadata : 1;
+	guint           has_fulltext : 1;
+	guint           has_thumbs : 1;
+	guint           show_service_files : 1;
+	guint           show_service_directories : 1;
 
 } ServiceDef;
 
@@ -193,19 +195,36 @@ typedef enum {
 	EVENT_NOTHING,
 	EVENT_SHUTDOWN,
 	EVENT_DISABLE,
+	EVENT_PAUSE,
 	EVENT_CACHE_FLUSHED
 } LoopEvent;
 
+
 typedef struct {
+	gchar	*uri;
+	time_t	first_change_time;
+	gint    num_of_change;
+} FileChange;
+
+
+typedef struct {
+
+	gboolean	readonly;
 
 	TrackerStatus	status;
 	int		pid;
 
+	gpointer	dbus_con;
+	gpointer	hal_con;
+
+	gboolean	reindex;
+
 	/* config options */
 	GSList 		*watch_directory_roots_list;
 	GSList 		*crawl_directory_list;
 	GSList 		*no_watch_directory_list;
-	GSList		*no_index_file_types;
+	GSList		*no_index_file_types_list;
+	GSList		*ignore_pattern_list;
 
 	gboolean	enable_indexing;
 	gboolean	enable_watching; /* can disable all forms of directory watching */
@@ -214,9 +233,7 @@ typedef struct {
 
 	guint32		watch_limit;
 
-	gboolean	shutdown;
-	gboolean	paused;
-	gboolean	battery_paused;
+
 
 	/* controls how much to output to screen/log file */
 	int		verbosity;
@@ -241,27 +258,36 @@ typedef struct {
 	int		optimization_count;   /* no of updates or inserts to be performed before optimizing hashtable */
 	int		throttle;
 	int		default_throttle;
-	int		battery_throttle;
 	gboolean	use_extra_memory;
 	int		initial_sleep;
 	int		max_words_to_index;
 	int 		memory_limit;
-	gboolean	battery_checking;
+	int 		thread_stack_size;
+	gboolean	fast_merges;
+
+	/* HAL battery */
+	char		*battery_udi;
+	gboolean	index_on_battery;
+	gboolean	initial_index_on_battery;
+
+	/* pause/shutdown vars */
+	gboolean	shutdown;
+	gboolean	pause_manual;
+	gboolean	pause_battery;
+	gboolean	pause_io;
+	gint		low_diskspace_limit;
 
 	/* indexing options */
-
-	/* indexing options */
-	int	 	max_index_bucket_count;
-	int	 	index_bucket_ratio; /* 0 = 50%, 1 = 100%, 2 = 200%, 3 = 300%, 4+ = 400% */
-	int		min_index_bucket_count;
-	int		index_divisions;
-	int 		padding; /* values 1-8 */
+	gint            max_index_bucket_count;
+	gint            min_index_bucket_count;
+	gint            index_bucket_ratio; /* 0 = 50%, 1 = 100%, 2 = 200%, 3 = 300%, 4+ = 400% */
+	gint            index_divisions;
+	gint            padding; /* values 1-8 */
 
 	gpointer	file_index;
 	gpointer	file_update_index;
 	gpointer	email_index;
 
-
 	guint32		merge_limit; 		/* size of index in MBs when merging is triggered -1 == no merging*/
 	gboolean	active_file_merge;
 	gboolean	active_email_merge;
@@ -282,7 +308,16 @@ typedef struct {
 	gboolean	first_time_index;
 	gboolean	first_flush;
 	gboolean	do_optimize;
+	
+	time_t		index_time_start;
+	int		folders_count;
+	int		folders_processed;
+	int		mbox_count;
+	int		mbox_processed;
 
+
+	const char	*current_uri;
+	
 	gboolean	skip_mount_points;	/* should tracker descend into mounted directories? see Tracker.root_directory_devices */
 	GSList *	root_directory_devices;
 
@@ -293,9 +328,6 @@ typedef struct {
 
 	char *		xesam_dir;
 
-	/* battery and ac power status file */
-	char		*battery_state_file;
-
 	/* service directory table - this is used to store a ServiceInfo struct for a directory path - used for determining which service a uri belongs to for things like files, emails, conversations etc*/
 	GHashTable	*service_directory_table;
 	GSList		*service_directory_list;
@@ -317,9 +349,20 @@ typedef struct {
 	/* nfs options */
 	gboolean	use_nfs_safe_locking; /* use safer but much slower external lock file when users home dir is on an nfs systems */
 
+	/* Queue for recorad file changes */
+	GQueue		*file_change_queue;
+	GSList		*tmp_black_list;
+	gboolean	black_list_timer_active;
+	
+	/* progress info for merges */
+	int		merge_count;
+	int		merge_processed;
+	
+
 	/* application run time values */
 	gboolean	is_indexing;
 	gboolean	in_flush;
+	gboolean	in_merge;
 	int		index_count;
 	int		index_counter;
 	int		update_count;
@@ -452,16 +495,13 @@ typedef struct {
 	char 			*uri;
 	guint32			file_id;
 
-	gboolean		is_new;
 	TrackerChangeAction  	action;
 	guint32        		cookie;
 	int  		     	counter;
 	FileTypes		file_type;
 	WatchTypes		watch_type;
-	gboolean		is_directory;
 
 	/* symlink info - File ID of link might not be in DB so need to store path/filename too */
-	gboolean                is_link;
 	gint32                  link_id;
 	char                    *link_path;
 	char                    *link_name;
@@ -476,14 +516,18 @@ typedef struct {
 	gint32                  offset;
 
 	/* options */
-	char			*move_from_uri;
-	gboolean		extract_embedded;
-	gboolean		extract_contents;
-	gboolean		extract_thumbs;
-	gboolean		is_hidden;
+	char			*moved_to_uri;
 
 	int			aux_id;
 
+	guint                   is_new : 1;
+	guint                   is_directory : 1;
+	guint                   is_link : 1;
+	guint                   extract_embedded : 1;
+	guint                   extract_contents : 1;
+	guint                   extract_thumbs : 1;
+	guint                   is_hidden : 1;
+
 	/* we ref count FileInfo as it has a complex lifespan and is tossed between various threads, lists, queues and hash tables */
 	int			ref_count;
 
@@ -614,10 +658,6 @@ void		tracker_free_metadata_field 	(FieldData *field_data);
 
 gboolean	tracker_unlink 			(const char *uri);
 
-char *		tracker_get_battery_state_file 	();
-gboolean	tracker_using_battery 		(void);
-
-
 int 		tracker_get_memory_usage 	(void);
 
 guint32		tracker_file_size 		(const char *name);
@@ -626,4 +666,13 @@ void		tracker_file_close 		(int fd, gboolean no_longer_needed);
 
 void		tracker_add_io_grace 		(const char *uri);
 
+char *		tracker_get_status 		(void);
+
+void		free_file_change		(FileChange **user_data);
+gboolean	tracker_do_cleanup 		(const gchar *sig_msg);
+
+gboolean	tracker_pause_on_battery 	(void);
+gboolean	tracker_low_diskspace		(void);
+gboolean	tracker_pause			(void);
+
 #endif
