@@ -29,6 +29,10 @@
 #include <config.h>
 #endif
 
+#ifdef OS_WIN32
+#include "../trackerd/mingw-compat.h"
+#endif
+
 #include "xdgmime.h"
 #include "xdgmimeint.h"
 #include "xdgmimeglob.h"
@@ -60,7 +64,7 @@ static XdgCallbackList *callback_list = NULL;
 XdgMimeCache **_caches = NULL;
 static int n_caches = 0;
 
-const char *xdg_mime_type_unknown = "application/octet-stream";
+const char xdg_mime_type_unknown[] = "application/octet-stream";
 
 
 enum
@@ -281,7 +285,8 @@ xdg_run_command_on_dirs (XdgDirectoryFunc  func,
  * FIXME: This doesn't protect against permission changes.
  */
 static int
-xdg_check_file (const char *file_path)
+xdg_check_file (const char *file_path,
+                int        *exists)
 {
   struct stat st;
 
@@ -290,6 +295,9 @@ xdg_check_file (const char *file_path)
     {
       XdgDirTimeList *list;
 
+      if (exists)
+        *exists = TRUE;
+
       for (list = dir_time_list; list; list = list->next)
 	{
 	  if (! strcmp (list->directory_name, file_path) &&
@@ -306,6 +314,9 @@ xdg_check_file (const char *file_path)
       return TRUE;
     }
 
+  if (exists)
+    *exists = FALSE;
+
   return FALSE;
 }
 
@@ -313,15 +324,30 @@ static int
 xdg_check_dir (const char *directory,
 	       int        *invalid_dir_list)
 {
-  int invalid;
+  int invalid, exists;
   char *file_name;
 
   assert (directory != NULL);
 
+  /* Check the mime.cache file */
+  file_name = malloc (strlen (directory) + strlen ("/mime/mime.cache") + 1);
+  strcpy (file_name, directory); strcat (file_name, "/mime/mime.cache");
+  invalid = xdg_check_file (file_name, &exists);
+  free (file_name);
+  if (invalid)
+    {
+      *invalid_dir_list = TRUE;
+      return TRUE;
+    }
+  else if (exists)
+    {
+      return FALSE;
+    }
+
   /* Check the globs file */
   file_name = malloc (strlen (directory) + strlen ("/mime/globs") + 1);
   strcpy (file_name, directory); strcat (file_name, "/mime/globs");
-  invalid = xdg_check_file (file_name);
+  invalid = xdg_check_file (file_name, NULL);
   free (file_name);
   if (invalid)
     {
@@ -332,18 +358,7 @@ xdg_check_dir (const char *directory,
   /* Check the magic file */
   file_name = malloc (strlen (directory) + strlen ("/mime/magic") + 1);
   strcpy (file_name, directory); strcat (file_name, "/mime/magic");
-  invalid = xdg_check_file (file_name);
-  free (file_name);
-  if (invalid)
-    {
-      *invalid_dir_list = TRUE;
-      return TRUE;
-    }
-
-  /* Check the mime.cache file */
-  file_name = malloc (strlen (directory) + strlen ("/mime/mime.cache") + 1);
-  strcpy (file_name, directory); strcat (file_name, "/mime/mime.cache");
-  invalid = xdg_check_file (file_name);
+  invalid = xdg_check_file (file_name, NULL);
   free (file_name);
   if (invalid)
     {
@@ -528,15 +543,15 @@ xdg_mime_get_mime_type_for_file (const char  *file_name,
 const char *
 xdg_mime_get_mime_type_from_file_name (const char *file_name)
 {
-  const char *mime_types[2];
+  const char *mime_type;
 
   xdg_mime_init ();
 
   if (_caches)
     return _xdg_mime_cache_get_mime_type_from_file_name (file_name);
 
-  if (_xdg_glob_hash_lookup_file_name (global_hash, file_name, mime_types, 2) == 1)
-    return mime_types[0];
+  if (_xdg_glob_hash_lookup_file_name (global_hash, file_name, &mime_type, 1))
+    return mime_type;
   else
     return XDG_MIME_TYPE_UNKNOWN;
 }
@@ -584,6 +599,17 @@ xdg_mime_shutdown (void)
       parent_list = NULL;
     }
   
+  if (_caches)
+    {
+      int i;
+
+      for (i = 0; i < n_caches; i++)
+        _xdg_mime_cache_unref (_caches[i]);
+      free (_caches);
+      _caches = NULL;
+      n_caches = 0;
+    }
+
   for (list = callback_list; list; list = list->next)
     (list->callback) (list->data);
 
@@ -601,7 +627,7 @@ xdg_mime_get_max_buffer_extents (void)
   return _xdg_mime_magic_get_buffer_extents (global_magic);
 }
 
-static const char *
+const char *
 _xdg_mime_unalias_mime_type (const char *mime_type)
 {
   const char *lookup;
@@ -663,7 +689,7 @@ xdg_mime_media_type_equal (const char *mime_a,
   return 0;
 }
 
-#if 0
+#if 1
 static int
 xdg_mime_is_super_type (const char *mime)
 {
@@ -696,7 +722,7 @@ _xdg_mime_mime_type_subclass (const char *mime,
   if (strcmp (umime, ubase) == 0)
     return 1;
 
-#if 0  
+#if 1  
   /* Handle supertypes */
   if (xdg_mime_is_super_type (ubase) &&
       xdg_mime_media_type_equal (umime, ubase))
